#include "string.h"

typedef bool INI_ConfigLoadParamsStringValidator(const char*);

void INI_ConfigEnsureValidString(INI_ConfigItem* item, const char* buffer) {
	if (item->validator) {
		INI_ConfigLoadParamsStringValidator* validator = item->validator;
		if (validator(buffer)) {
			memcpy(item->dest, buffer, item->type.size);
			return;
		}
		memcpy(item->dest, item->default_value, item->type.size);
		return;
	}
	memcpy(item->dest, buffer, item->type.size);
}

void INI_ConfigLoadString(INI_ConfigItem* item, const char* config_path, void* item_array) {
	char buffer[item->type.size];
	ini_gets(item->section, item->key, item->default_value, buffer, (int32_t) item->type.size, config_path);
	INI_ConfigEnsureValidString(item, buffer);
}

INI_ConfigSaveItemResult INI_ConfigSaveString(INI_ConfigItem* item, const char* config_path, void* item_array) {
	const char* dest = (const char*) item->dest;
	if (item->validator) {
		INI_ConfigLoadParamsStringValidator* validator = item->validator;
		if (!validator(dest)) {
			dest = (const char*) item->default_value;
		}
	}

	if (!ini_puts(item->section, item->key, dest, config_path)) {
		return INI_CONFIG_SAVE_ITEM_ERROR;
	}

	return INI_CONFIG_SAVE_ITEM_OK;
}

void INI_ConfigOverrideString(INI_ConfigItem* item, uint8_t argc, const char* argv[]) {
	if (!item->cmdline_option) {
		return;
	}
	const char* cmdline_buffer;
	if ((cmdline_buffer = INI_ConfigGetCommandlineOption(argc, argv, item->cmdline_option))) {
		INI_ConfigEnsureValidString(item, cmdline_buffer);
	}
}
